/*
 * Copyright 2004-2006 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.extension.dxo.converter.impl;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Set;

import org.seasar.extension.dxo.converter.Converter;
import org.seasar.extension.dxo.converter.ConverterFactory;
import org.seasar.framework.util.ClassUtil;

/**
 * @author koichik
 * 
 */
public class ConverterFactoryImpl implements ConverterFactory {

    protected static final Class[][] CONVERTERS = new Class[][] {
            { Boolean.class, BooleanConverter.class },
            { Byte.class, ByteConverter.class },
            { Short.class, ShortConverter.class },
            { Integer.class, IntegerConverter.class },
            { Long.class, LongConverter.class },
            { Float.class, FloatConverter.class },
            { Double.class, DoubleConverter.class },
            { BigDecimal.class, BigDecimalConverter.class },
            { BigInteger.class, BigIntegerConverter.class },
            { Character.class, CharacterConverter.class },
            { String.class, StringConverter.class },
            { Date.class, DateConverter.class },
            { java.sql.Date.class, SqlDateConverter.class },
            { java.sql.Time.class, SqlTimeConverter.class },
            { java.sql.Timestamp.class, SqlTimestampConverter.class },
            { Calendar.class, CalendarConverter.class },
            { Object[].class, ArrayConverter.class },
            { char[].class, CharArrayConverter.class },
            { List.class, ListConverter.class },
            { Set.class, SetConverter.class }, };

    public Converter getConverter(final Class sourceClass, final Class destClass) {
        final Class keyClass = ClassUtil
                .getWrapperClassIfPrimitive(destClass == Object.class ? sourceClass
                        : destClass);
        for (int i = 0; i < CONVERTERS.length; ++i) {
            if (keyClass.isAssignableFrom(CONVERTERS[i][0])) {
                return (Converter) ClassUtil.newInstance(CONVERTERS[i][1]);
            }
        }
        return new BeanConverter();
    }

}
